// ==============================================================
//
// solar_probe_2005.cpp
// Control module for solar_probe_2005 vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================



const double ISP = 2150;
const double RCSTH = 25;
const double FUELMASS = 60;
const double DRYMASS = 612;


const double MAST_SPEED = 0.1;
const double SOLP_SPEED = 0.1;
const double LID_SPEED = 0.1;
const double BOOM_SPEED = 0.1;
const double MAX_HEATER = 300;
const double MAX_RADIATOR = -500;
const double MAX_RADIATOR_TPS = 0;
const double RTG_INPUT = 90;
const double SPEC_HEAT_CAP = 750;
const double TPS_MASS = 134;
const double HULL_MASS = 478;
const double StefBolt = 5.67e-8;

#define GREEN RGB(125, 255, 125)

void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.03 + 0.4*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
}

void HLiftCoeff (VESSEL *v, double beta, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.03 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}
// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class solar_probe_2005: public VESSEL3 {
public:
	solar_probe_2005 (OBJHANDLE hVessel, int flightmodel);
	~solar_probe_2005 ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkPostCreation();
	void DefineAnimations (void);
	void DoThermal(void);
	void DoBoom(void);
	
	
	THRUSTER_HANDLE th_rcs[12], th_group[6], th_boom;
	PROPELLANT_HANDLE hProp;

	ANIMATIONCOMPONENT_HANDLE Section, Section1, Section2, Section3, Section4, Section5;

	


private:

	double step_time;
	int kr_status, solp_status, boom_status, lid_status, mast_status, asp_status, per_status;


	UINT anim_solp, anim_boom, anim_lid, anim_mast, anim_per;
	double solp_proc, boom_proc, lid_proc, mast_proc, per_proc;
	double hull_temp, tps_temp;
	double req_power_hull, req_power_tps;
	double bus_th, tps_th;
	double solflux;
	double heat_input_hull, heat_output_hull, heat_input_tps, heat_output_tps;
	double ps_bus_tps, dtemp_hull, dtemp_tps;
	double old_hull_temp, old_tps_temp, hud_dtemp_hull, hud_dtemp_tps;
	double hull_psv_tf, tps_psv_tf;
	double hud_upd_timer, per_timer;
	int boom_sw;

	
	MESHHANDLE mesh12;
	DEVMESHHANDLE dhMesh2;

	void SetMeshMat(void);
	void SetBusMeshMat(void);
	void DoSunPointing(void);

	
};

solar_probe_2005::solar_probe_2005 (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

solar_probe_2005::~solar_probe_2005 ()
{
}

void solar_probe_2005::DefineAnimations (void)
{
// 1. SOLP/ ant

	anim_solp = CreateAnimation (0);
	static UINT Solp1Grp[2] = {25,26};
	static MGROUP_ROTATE Solp1 (0, Solp1Grp, 2, _V(0,0.59,-0.255), _V(1,0,0), (float)(-60*RAD));
	
	static UINT Solp2Grp[3] = {27,28,29};
	static MGROUP_ROTATE Solp2 (0, Solp2Grp, 3, _V(0,0.737,0.022), _V(1,0,0), (float)(110*RAD));


	Section1 = AddAnimationComponent (anim_solp, 0, 1, &Solp1);
	AddAnimationComponent (anim_solp, 0, 1, &Solp2, Section1);


// Boom / boom

	anim_boom = CreateAnimation (0);
	static UINT AppArm1Grp[1] = {12};
	static MGROUP_TRANSLATE AppArm1 (0, AppArm1Grp, 1, _V(0,0,-0.8));
	
	static UINT AppArm2Grp[3] = {9,10,11};
	static MGROUP_TRANSLATE AppArm2 (0, AppArm2Grp, 3, _V(0,0,-1.6));

	static UINT AppArm3Grp[2] = {7,8};
	static MGROUP_TRANSLATE AppArm3 (0, AppArm3Grp, 2, _V(0,0,-2.4));

	AddAnimationComponent (anim_boom, 0, 1, &AppArm1);
	AddAnimationComponent (anim_boom, 0, 1, &AppArm2);
	AddAnimationComponent (anim_boom, 0, 1, &AppArm3);


// lid / fpi
	anim_lid = CreateAnimation (0);
	
	static UINT TopArm1Grp[2] = {3,19};
	static MGROUP_ROTATE TopArm1 (0, TopArm1Grp, 2, _V(-0.735,0,-0.294), _V(0,1,0), (float)(60*RAD));
	
	static UINT TopArm2Grp[2] = {2,4};
	static MGROUP_ROTATE TopArm2 (0, TopArm2Grp, 2, _V(-0.73,0,-0.647), _V(0,1,0), (float)(-25*RAD));
	

	Section2 = AddAnimationComponent (anim_lid, 0, 1, &TopArm1);
	AddAnimationComponent (anim_lid, 0, 1, &TopArm2,Section2);


// masts
	anim_mast = CreateAnimation (0);
	static UINT Ant1Grp[1] = {22};
	static MGROUP_ROTATE Ant1 (0, Ant1Grp, 1, _V(0.681,0.023,-1.287), _V(-0.259,0.966,0), (float)(165*RAD));
	
	static UINT Ant2Grp[1] = {21};
	static MGROUP_ROTATE Ant2 (0, Ant2Grp, 1, _V(-0.36,0.577,-1.287), _V(-0.7071,-0.7071,0), (float)(165*RAD));

	static UINT Ant3Grp[1] = {20};
	static MGROUP_ROTATE Ant3 (0, Ant3Grp, 1, _V(-0.32,-0.601,-1.287), _V(0.966,-0.259,0), (float)(165*RAD));

	
	AddAnimationComponent (anim_mast, 0, 1, &Ant1);
	AddAnimationComponent (anim_mast, 0, 1, &Ant2);
	AddAnimationComponent (anim_mast, 0, 1, &Ant3);


	//periscope
	anim_per = CreateAnimation (0);
	static UINT Per1Grp[3] = {15,16,17};
	static MGROUP_ROTATE Per1 (0, Per1Grp, 3, _V(0.588,-0.258,-0.609), _V(0.5,0.866,0), (float)(-90*RAD));
	
	AddAnimationComponent (anim_per, 0, 1, &Per1);



}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void solar_probe_2005::clbkSetClassCaps (FILEHANDLE cfg)
{
	step_time = 0;
	kr_status = 0;

	solp_status = 0;
	boom_status = 0;
	lid_status = 0;
	mast_status = 0;
	asp_status = 0;
	per_status = 0;

	solp_proc = 0.0;
	boom_proc = 0.0;
	lid_proc = 0.0;
	mast_proc = 0.0;
	per_proc = 0.0;
	bus_th = 0.0;
	tps_th = 0.0;
	solflux = 0.0;
	boom_sw = 0;

	hull_temp = 293;
	tps_temp = 293;
	req_power_hull = 0;
	req_power_tps = 0;

	heat_input_hull = 0;
	heat_output_hull = 0;
	heat_input_tps = 0;
	heat_output_tps = 0;
	ps_bus_tps = 0;
	dtemp_hull = 0;
	dtemp_tps = 0;
	old_hull_temp = 0;
	old_tps_temp = 0;
	hud_dtemp_hull = 0;
	hud_dtemp_tps = 0;
	hull_psv_tf = 0;
	tps_psv_tf = 0;

	hud_upd_timer = 1.0;
	per_timer = 0;
	
	
	DefineAnimations ();
	// physical vessel parameters
	SetSize (6);
	SetEmptyMass (DRYMASS);
	SetPMI(_V(5.9,5.9,0.66));
	SetCrossSections (_V(11.2,11.9,6.6));
	SetCW(1,1,1,1);
	SetRotDrag (_V(2,2,1));
	SetTouchdownPoints ( _V(0,-1, 5), _V(-2,-1,-5), _V( 2,-1,-5));

	// camera parameters
	SetCameraOffset (_V(0,2,-2));

	//SetMeshVisibilityMode(AddMesh("SolarProbe/solar_probe_2005"),MESHVIS_ALWAYS);

	mesh12 = oapiLoadMesh ("SolarProbe2005/solar_probe_2005");
	AddMesh(mesh12);
	SetMeshVisibilityMode (0, MESHVIS_EXTERNAL);

	

	// propellant resources
	
	hProp = CreatePropellantResource (FUELMASS,FUELMASS);

	// ***************** thruster definitions *******************
	

	th_rcs[0] = CreateThruster (_V( 0.265,-0.561,-1.347), _V( 0.7071, 0.7071, 0), RCSTH, hProp, ISP);
	th_rcs[1] = CreateThruster (_V(-0.265,-0.561,-1.347), _V(-0.7071, 0.7071, 0), RCSTH, hProp, ISP);
	th_rcs[2] = CreateThruster (_V( 0.265, 0.561,-1.347), _V( 0.7071,-0.7071, 0), RCSTH, hProp, ISP);
	th_rcs[3] = CreateThruster (_V(-0.265, 0.561,-1.347), _V(-0.7071,-0.7071, 0), RCSTH, hProp, ISP);

	th_rcs[4] = CreateThruster (_V( 0.352,-0.509,-1.347), _V(-0.966,-0.259, 0), RCSTH, hProp, ISP);
	th_rcs[5] = CreateThruster (_V(-0.352,-0.509,-1.347), _V( 0.966,-0.259, 0), RCSTH, hProp, ISP);
	th_rcs[6] = CreateThruster (_V( 0.352, 0.509,-1.347), _V(-0.966, 0.259, 0), RCSTH, hProp, ISP);
	th_rcs[7] = CreateThruster (_V(-0.352, 0.509,-1.347), _V( 0.966, 0.259, 0), RCSTH, hProp, ISP);
	
	th_rcs[8] = CreateThruster (_V( 0,-0.557,-1.429), _V(0,0,1), RCSTH, hProp, ISP);
	th_rcs[9] = CreateThruster (_V( 0, 0.557,-1.429), _V(0,0,1), RCSTH, hProp, ISP);
	th_rcs[10] = CreateThruster (_V(-0.557, 0,-1.429), _V(0,0,1), RCSTH, hProp, ISP);
	th_rcs[11] = CreateThruster (_V( 0.557, 0,-1.429), _V(0,0,1), RCSTH, hProp, ISP);
	
	th_boom = CreateThruster (_V( 0.557, 0.3,-0.429), _V(0.577,0.577,0.577), 1000, hProp, 100);


	th_group[0] = th_rcs[2];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHUP);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[1];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_PITCHDOWN);

	th_group[0] = th_rcs[4];
	th_group[1] = th_rcs[6];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[5];
	th_group[1] = th_rcs[7];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	th_group[0] = th_rcs[1];
	th_group[1] = th_rcs[2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKRIGHT);

	th_group[0] = th_rcs[0];
	th_group[1] = th_rcs[3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_BANKLEFT);

	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[9];
	th_group[2] = th_rcs[10];
	th_group[3] = th_rcs[11];
	CreateThrusterGroup (th_group, 4, THGROUP_ATT_FORWARD);


	th_group[0] = th_rcs[8];
	th_group[1] = th_rcs[9];
	th_group[2] = th_rcs[10];
	th_group[3] = th_rcs[11];
	CreateThrusterGroup (th_group, 4, THGROUP_MAIN);

	SURFHANDLE exhaust = oapiRegisterExhaustTexture ("exhaust_atrcs");


	double exlen = 0.5;
	double exwid = 0.05;

	AddExhaust(th_rcs[0],exlen,exwid,exhaust);
	AddExhaust(th_rcs[1],exlen,exwid,exhaust);
	AddExhaust(th_rcs[2],exlen,exwid,exhaust);
	AddExhaust(th_rcs[3],exlen,exwid,exhaust);
	AddExhaust(th_rcs[4],exlen,exwid,exhaust);
	AddExhaust(th_rcs[5],exlen,exwid,exhaust);
	AddExhaust(th_rcs[6],exlen,exwid,exhaust);
	AddExhaust(th_rcs[7],exlen,exwid,exhaust);
	AddExhaust(th_rcs[8],exlen,exwid,exhaust);
	AddExhaust(th_rcs[9],exlen,exwid,exhaust);
	AddExhaust(th_rcs[10],exlen,exwid,exhaust);
	AddExhaust(th_rcs[11],exlen,exwid,exhaust);

	PARTICLESTREAMSPEC exhaust_main = {
		0, 2, 5, 10, 0.7, 15, 20, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 8,
		PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
	};

	AddExhaustStream (th_boom,&exhaust_main);


	///////////////////////////////////////////////////////
	/////AirFoils


	CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,1), VLiftCoeff, 0, 1, 2, 1);
	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,1), HLiftCoeff, 0, 1, 2, 1);
}



// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new solar_probe_2005 (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (solar_probe_2005*)vessel;
}

void solar_probe_2005::clbkVisualCreated (VISHANDLE vis, int refcount)
{
dhMesh2 = GetDevMesh(vis, 0);
SetMeshMat();
}



void solar_probe_2005::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%d %0.4f %d %0.4f %d %0.4f", solp_status, solp_proc, boom_status, boom_proc, per_status, per_proc);
	oapiWriteScenario_string (scn, "ANTS", cbuf);

	sprintf (cbuf, "%d %0.4f %d %0.4f", lid_status, lid_proc, mast_status, mast_proc);
	oapiWriteScenario_string (scn, "FPMA", cbuf);
	
	sprintf (cbuf, "%d %d %d %0.4f %0.4f",boom_sw, kr_status, asp_status, hull_temp, tps_temp);
	oapiWriteScenario_string (scn, "KRTP", cbuf);


}
void solar_probe_2005::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ANTS", 4)) {
			sscanf (line+4, "%d%lf%d%lf%d%lf", &solp_status, &solp_proc, &boom_status, &boom_proc, &per_status, &per_proc);
		} else if (!strnicmp (line, "FPMA", 4)) {
			sscanf (line+4, "%d%lf%d%lf", &lid_status, &lid_proc, &mast_status, &mast_proc);
		} else if (!strnicmp (line, "KRTP", 4)) {
			sscanf (line+4, "%d%d%d%lf%lf",&boom_sw, &kr_status, &asp_status, &hull_temp, &tps_temp);
		}else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }

	SetAnimation(anim_solp, solp_proc);
	SetAnimation(anim_boom, boom_proc);
	SetAnimation(anim_lid, lid_proc);
	SetAnimation(anim_mast, mast_proc);



}



int solar_probe_2005::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

	if (boom_sw == 1)return 0;

  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
		

	case OAPI_KEY_NUMPAD5: // killrot
		if (kr_status == 0)
		{
		asp_status = 0;
		kr_status = 1;
		}
		else if (kr_status == 1) kr_status = 0;
		return 1;

	} 
  }

  else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) {

		case OAPI_KEY_G: // hg ant
		if (solp_status == 0) solp_status = 1;
		return 1;

		case OAPI_KEY_K: // boom and masts and fpi
		if (boom_status == 0) boom_status = 1;
		if (mast_status == 0) mast_status = 1;
		if (lid_status == 0) lid_status = 1;
		return 1;

		case OAPI_KEY_V: // per shot
		if (per_status == 0) per_status = 1;
		else if (per_status > 0)per_status = 3;
		return 1;

		case OAPI_KEY_B: // auto sun pointing
		if (asp_status == 0)
			{		
			kr_status = 0;
			asp_status = 1;
			}
		else asp_status = 0;
		return 1;


	}
  }
  return 0;
}


int solar_probe_2005::clbkConsumeDirectKey(char *kstate)
{
	if (boom_sw == 1)return 0;

		VECTOR3 avel = _V(0,0,0);
		GetAngularVel(avel);
		double dt = oapiGetSimStep();
		double drot = RAD*2*dt;

if(kr_status == 0)
{
if (KEYMOD_SHIFT (kstate))
	{
if (KEYDOWN (kstate, OAPI_KEY_NUMPAD2))
		{
		if (avel.x < RAD*10)
			{
			avel.x = avel.x+drot;
			if (avel.x > RAD*10) avel.x = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD8))
		{
		if (avel.x > -RAD*10)
			{
			avel.x = avel.x-drot;
			if (avel.x < -RAD*10) avel.x = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD1))
		{
		if (avel.y < RAD*10)
			{
			avel.y = avel.y+drot;
			if (avel.y > RAD*10) avel.y = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD3))
		{
		if (avel.y > -RAD*10)
			{
			avel.y = avel.y-drot;
			if (avel.y < -RAD*10) avel.y = -RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD6))
		{
		if (avel.z < RAD*10)
			{			
			avel.z = avel.z+drot;
			if (avel.z > RAD*10) avel.z = RAD*10;
			}
		}

if (KEYDOWN (kstate, OAPI_KEY_NUMPAD4))
		{
		if (avel.z > -RAD*10)
			{
			avel.z = avel.z-drot;
			if (avel.z < -RAD*10) avel.z = -RAD*10;
			}		
		}

		SetAngularVel(avel);
	}	
}

return 0;
}


void solar_probe_2005::clbkPostCreation()
{
	old_hull_temp = hull_temp;
	old_tps_temp = tps_temp;
	if (boom_sw == 1) boom_sw = 2;
}



void solar_probe_2005::clbkPreStep(double SimT, double SimDT, double MJD)
{

step_time = SimDT;///////
double simdt = SimDT;


//sprintf(oapiDebugString(), "%d %0.4f %0.4f", (sa1_stbd_elb_sts + sa1_stbd_scale_sts + sa1_port_elb_sts + sa1_port_scale_sts),sa1_port_rot_proc,sa1_stbd_rot_proc);

if(kr_status == 1)
{
VECTOR3 avel = _V(0,0,0);
GetAngularVel(avel);
double drot = RAD*2*simdt;

if (avel.x > -drot && avel.x < drot) avel.x = 0;
else if (avel.x > drot) avel.x = avel.x - drot;
else if (avel.x < -drot) avel.x = avel.x + drot;

if (avel.y > -drot && avel.y < drot) avel.y = 0;
else if (avel.y > drot) avel.y = avel.y - drot;
else if (avel.y < -drot) avel.y = avel.y + drot;

if (avel.z > -drot && avel.z < drot) avel.z = 0;
else if (avel.z > drot) avel.z = avel.z - drot;
else if (avel.z < -drot) avel.z = avel.z + drot;

if ((avel.x + avel.y + avel.z) == 0)kr_status = 0;

SetAngularVel(avel);
}



////////////////// ANIMATIONS ///////////////

char cbuf[256]="Sun";
	OBJHANDLE hRef = oapiGetGbodyByName (cbuf);
	VECTOR3 sun_pos_global = _V(0,0,0);
	oapiGetGlobalPos(hRef, &sun_pos_global);

	VECTOR3 sun_dir_local = _V(0,0,0);
	Global2Local(sun_pos_global,sun_dir_local);
	double sun_dist = length(sun_dir_local);
	double sun_radius = oapiGetSize(hRef);
	double umbra_angle = atan2(sun_radius, sun_dist);
	umbra_angle = umbra_angle*DEG;




// solp
if (solp_status == 1)
{
double da = SimDT * SOLP_SPEED;

double req_deflection = 1-(umbra_angle/14);
if (req_deflection > 1) req_deflection = 1;
if (req_deflection < 0) req_deflection = 0;

if (solp_proc < req_deflection)
	{
	solp_proc= solp_proc+da;
	if (solp_proc > req_deflection)solp_proc = req_deflection;
	}
else if (solp_proc > req_deflection)
	{
	solp_proc= solp_proc-da;
	if (solp_proc < req_deflection)solp_proc = req_deflection;
	}

SetAnimation(anim_solp, solp_proc);
}






// boom
if (boom_status == 1)
{
double da = SimDT * BOOM_SPEED;

boom_proc = boom_proc+da;
if (boom_proc > 1)
	{
	boom_proc = 1;
	boom_status = 2;
	}
SetAnimation(anim_boom, boom_proc);	
}

else if (boom_status == 2)SetAnimation(anim_boom, 1);	




// masts
if (mast_status == 1)
{
double da = SimDT * MAST_SPEED;

double req_deflection = (109+(4*umbra_angle))/165;
if (req_deflection > 1) req_deflection = 1;

if (mast_proc < req_deflection)
	{
	mast_proc= mast_proc+da;
	if (mast_proc > req_deflection)mast_proc = req_deflection;
	}
else if (mast_proc > req_deflection)
	{
	mast_proc= mast_proc-da;
	if (mast_proc < req_deflection)mast_proc = req_deflection;
	}
	SetAnimation(anim_mast, mast_proc);	
}

//else if (mast_status == 2)SetAnimation(anim_mast, 1);	




/// lid
if (lid_status == 1)
{
double da = SimDT * LID_SPEED;

double req_deflection = 1-(umbra_angle/28);
if (req_deflection > 1) req_deflection = 1;
if (req_deflection < 0) req_deflection = 0;

if (lid_proc < req_deflection)
	{
	lid_proc= lid_proc+da;
	if (lid_proc > req_deflection)lid_proc = req_deflection;
	}
else if (lid_proc > req_deflection)
	{
	lid_proc= lid_proc-da;
	if (lid_proc < req_deflection)lid_proc = req_deflection;
	}
SetAnimation(anim_lid, lid_proc);	
}


////////// periscope
if (per_status == 1)
{
double da = SimDT * SOLP_SPEED;

per_proc = per_proc+da;
if (per_proc > 1)
	{
	per_proc = 1;
	per_status = 2;
	}
SetAnimation(anim_per, per_proc);
}

else if (per_status == 2)
{
per_timer = per_timer+SimDT;
if (per_timer > 10)
	{
	per_timer = 0;
	per_status = 3;
	}
SetAnimation(anim_per, 1);
}

else if (per_status == 3)
{
double da = SimDT * SOLP_SPEED;

per_proc = per_proc-da;
if (per_proc < 0)
	{
	per_proc = 0;
	per_status = 0;
	}
SetAnimation(anim_per, per_proc);
}





 if (10001 > oapiGetTimeAcceleration ())DoThermal();
	//DoThermal();

	if (asp_status == 1) DoSunPointing();


}

void solar_probe_2005::clbkPostStep (double simt, double simdt, double mjd)
{
	if (tps_temp > 800) SetMeshMat();
	if (hull_temp > 800) SetBusMeshMat();
	if (hull_temp > 573 && boom_sw == 0) DoBoom();
	if (boom_sw == 2)DoBoom();
}




bool solar_probe_2005::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	int w10 = (int)(344*d);
	int w11 = (int)(360*d);
	int w12 = (int)(376*d);
	int w13 = (int)(392*d);
	int w14 = (int)(408*d);
	int w15 = (int)(424*d);

	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Solar Flux %0.0f W/m", solflux);
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	double m1 = GetEmptyMass();
	double m2 = GetPropellantMass(hProp);
	double test = ((m1+m2)/m1);
	double dv = (ISP*(log(test)));

	sprintf(abuf,"Remaining dV = %0.2f m/s", dv);
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Reaction Wheel Att.Ctrl. = SHIFT+NUMPAD");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	if (kr_status == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"RW KillRot Active");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	if (asp_status == 1)
	{
	skp->SetTextColor(0x0066FFFF);
	char abuf[256];
	sprintf(abuf,"Auto Sun Pointing Active");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}

	if (asp_status == 0 && kr_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"B = Auto Sun Pointing");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}



	if (solp_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"G = Deploy HGA");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}


	if (boom_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"K = Deploy Mag.Boom FPI EF Ant.");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	if (per_status == 0)
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"V = Deploy Periscope");
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}

	/////// bus temp
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Bus Temp");
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

	{
	if(hull_temp < 283) skp->SetTextColor(0x00FF6666);
	else if(hull_temp > 313) skp->SetTextColor(0x006666FF);
	else skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"         %0.1f C", hull_temp-273);
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}

//// bus temp/sec

	{
	char abuf[256];
	if(hud_dtemp_hull < 0)
		{
		skp->SetTextColor(0x00FF6666);
		sprintf(abuf,"                     %0.3f C/s", hud_dtemp_hull);
		}
	else if(hud_dtemp_hull > 0)
		{
		skp->SetTextColor(0x006666FF);
		sprintf(abuf,"                     +%0.3f C/s", hud_dtemp_hull);
		}
	else 
		{
		skp->SetTextColor(0x0066FF66);
		sprintf(abuf,"                     +%0.3f C/s", hud_dtemp_hull);
		}
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}


	/// passive heatflow
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Bus Passive Heatflow");
	skp->Text ((10+roxl), (w6+royl), abuf,strlen(abuf));
	}

	{
		skp->SetTextAlign(skp->RIGHT,skp->TOP);
	if(hull_psv_tf < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"%0.0f W", hull_psv_tf);
		skp->Text ((10+roxl)+(chw*32), (w6+royl), abuf,strlen(abuf));
		}
	else if(hull_psv_tf > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", hull_psv_tf);
		skp->Text ((10+roxl)+(chw*32), (w6+royl), abuf,strlen(abuf));
		}
	else
		{
		skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", hull_psv_tf);
		skp->Text ((10+roxl)+(chw*32), (w6+royl), abuf,strlen(abuf));
		}
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}

	/// bus t control
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Bus Thermal Ctrl.");
	skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
	}

	{
	if(req_power_hull < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"                                 RADIATORS");
		skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
		}
	else if(req_power_hull > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"                                 HEATERS");
		skp->Text ((10+roxl), (w7+royl), abuf,strlen(abuf));
		}
	}

	{
		skp->SetTextAlign(skp->RIGHT,skp->TOP);
	if(req_power_hull < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"%0.0f W", req_power_hull);
		skp->Text ((10+roxl)+(chw*32), (w7+royl), abuf,strlen(abuf));
		}
	else if(req_power_hull > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", req_power_hull);
		skp->Text ((10+roxl)+(chw*32), (w7+royl), abuf,strlen(abuf));
		}
	else
		{
		skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", req_power_hull);
		skp->Text ((10+roxl)+(chw*32), (w7+royl), abuf,strlen(abuf));
		}
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}

	/// bus tps heat trans
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Bus->TPS Heat Trans");
	skp->Text ((10+roxl), (w8+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextAlign(skp->RIGHT,skp->TOP);
	char abuf[256];
	if(ps_bus_tps < 0)
	{
		skp->SetTextColor(0x00FF6666);
		sprintf(abuf,"%0.0f W", ps_bus_tps);
	}
	else if(ps_bus_tps > 0)
	{
		skp->SetTextColor(0x006666FF);
		sprintf(abuf,"+%0.0f W", ps_bus_tps);
	}
	else
	{
		skp->SetTextColor(0x0066FF66);
		sprintf(abuf,"+%0.0f W", ps_bus_tps);
	}
	skp->Text ((10+roxl)+(chw*32), (w8+royl), abuf,strlen(abuf));
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}



	//// bus total
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"Bus Total Heating");
	skp->Text ((10+roxl), (w9+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextAlign(skp->RIGHT,skp->TOP);
	if(bus_th < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"%0.0f W", bus_th);
		skp->Text ((10+roxl)+(chw*32), (w9+royl), abuf,strlen(abuf));
		}
	else if(bus_th > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", bus_th);
		skp->Text ((10+roxl)+(chw*32), (w9+royl), abuf,strlen(abuf));
		}
	else
		{
		skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", bus_th);
		skp->Text ((10+roxl)+(chw*32), (w9+royl), abuf,strlen(abuf));
		}
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}

	


	/// tps temp
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"TPS Temp");
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}

	{
	if(tps_temp < 273) skp->SetTextColor(0x00FF6666);
	else if(tps_temp > 2313) skp->SetTextColor(0x006666FF);
	else skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"         %0.1f C", tps_temp-273);
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}
	//// tps temp rate


	{
	char abuf[256];
	if(hud_dtemp_tps < 0)
		{
		skp->SetTextColor(0x00FF6666);
		sprintf(abuf,"                     %0.3f C/s", hud_dtemp_tps);
		}
	else if(hud_dtemp_tps > 0)
		{
		skp->SetTextColor(0x006666FF);
		sprintf(abuf,"                     +%0.3f C/s", hud_dtemp_tps);
		}
	else 
		{
		skp->SetTextColor(0x0066FF66);
		sprintf(abuf,"                     +%0.3f C/s", hud_dtemp_tps);
		}
	skp->Text ((10+roxl), (w11+royl), abuf,strlen(abuf));
	}

	/// passive heatflow
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"TPS Passive Heatflow");
	skp->Text ((10+roxl), (w12+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextAlign(skp->RIGHT,skp->TOP);
	if(tps_psv_tf < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"%0.0f W", tps_psv_tf);
		skp->Text ((10+roxl)+(chw*32), (w12+royl), abuf,strlen(abuf));
		}
	else if(tps_psv_tf > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", tps_psv_tf);
		skp->Text ((10+roxl)+(chw*32), (w12+royl), abuf,strlen(abuf));
		}
	else
		{
		skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", tps_psv_tf);
		skp->Text ((10+roxl)+(chw*32), (w12+royl), abuf,strlen(abuf));
		}
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}

	///// tps thermal control
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"TPS Thermal Ctrl.");
	skp->Text ((10+roxl), (w13+royl), abuf,strlen(abuf));
	}

	{
	if(req_power_tps < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"                                 RADIATORS");
		skp->Text ((10+roxl), (w13+royl), abuf,strlen(abuf));
		}
	else if(req_power_tps > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"                                 HEATERS");
		skp->Text ((10+roxl), (w13+royl), abuf,strlen(abuf));
		}
	}

	{
	skp->SetTextAlign(skp->RIGHT,skp->TOP);
	if(req_power_tps < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"%0.0f W", req_power_tps);
		skp->Text ((10+roxl)+(chw*32), (w13+royl), abuf,strlen(abuf));
		}
	else if(req_power_tps > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", req_power_tps);
		skp->Text ((10+roxl)+(chw*32), (w13+royl), abuf,strlen(abuf));
		}
	else
		{
		skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", req_power_tps);
		skp->Text ((10+roxl)+(chw*32), (w13+royl), abuf,strlen(abuf));
		}
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}

	/// tps bus heat trans
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"TPS->Bus Heat Trans");
	skp->Text ((10+roxl), (w14+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextAlign(skp->RIGHT,skp->TOP);
	char abuf[256];
	if(-ps_bus_tps < 0)
		{
		skp->SetTextColor(0x00FF6666);
		sprintf(abuf,"%0.0f W", -ps_bus_tps);
		}
	else if(-ps_bus_tps > 0)
		{
		skp->SetTextColor(0x006666FF);
		sprintf(abuf,"+%0.0f W", -ps_bus_tps);
		}
	else
		{
		skp->SetTextColor(0x0066FF66);
		sprintf(abuf,"+%0.0f W", ps_bus_tps);
		}
	skp->Text ((10+roxl)+(chw*32), (w14+royl), abuf,strlen(abuf));
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}
	

	////////// tps total
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"TPS Total Heating");
	skp->Text ((10+roxl), (w15+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextAlign(skp->RIGHT,skp->TOP);
	if(tps_th < 0)
		{
		skp->SetTextColor(0x00FF6666);
		char abuf[256];
		sprintf(abuf,"%0.0f W", tps_th);
		skp->Text ((10+roxl)+(chw*32), (w15+royl), abuf,strlen(abuf));
		}
	else if(tps_th > 0)
		{
		skp->SetTextColor(0x006666FF);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", tps_th);
		skp->Text ((10+roxl)+(chw*32), (w15+royl), abuf,strlen(abuf));
		}
	else
		{
		skp->SetTextColor(0x0066FF66);
		char abuf[256];
		sprintf(abuf,"+%0.0f W", tps_th);
		skp->Text ((10+roxl)+(chw*32), (w15+royl), abuf,strlen(abuf));
		}
	skp->SetTextAlign(skp->LEFT,skp->TOP);
	}

	
	return true;
}

void solar_probe_2005::DoThermal(void)
{
	char cbuf[256]="Sun";
	OBJHANDLE hRef = oapiGetGbodyByName (cbuf);
	VECTOR3 sun_pos_global = _V(0,0,0);
	oapiGetGlobalPos(hRef, &sun_pos_global);


	//VECTOR3 rsun = _V(0,0,0);
	VECTOR3 rsunloc= _V(0,0,0);
	Global2Local(sun_pos_global,rsunloc); // convert to vessel frame
	double sun_dist = length(rsunloc);
	//normalise(rsunloc);

	rsunloc.x = rsunloc.x/sun_dist;
	rsunloc.y = rsunloc.y/sun_dist;
	rsunloc.z = rsunloc.z/sun_dist;

	VECTOR3 x_axis = _V(1,0,0);
	VECTOR3 y_axis = _V(0,1,0);
	VECTOR3 z_axis = _V(0,0,1);

	double cos_sunangle_x = dotp(x_axis, rsunloc);
	double cos_sunangle_y = dotp(y_axis, rsunloc);
	double cos_sunangle_z = dotp(z_axis, rsunloc);

	solflux = (AU/sun_dist)*(AU/sun_dist)*1380;

	//// heating
	double hull_absorbtion = 0.075;
	/// original 0.15
	double tps_absorbtion = 0.075;
	int z_sw = 0;
	if (rsunloc.z < 0) z_sw = 1;

	if (cos_sunangle_x < 0)cos_sunangle_x= -cos_sunangle_x;
	if (cos_sunangle_y < 0)cos_sunangle_y= -cos_sunangle_y;
	if (cos_sunangle_z < 0)cos_sunangle_z= -cos_sunangle_z;

	double sa = acos(cos_sunangle_z);
	double sin_sa = sin(sa);


	double cone_umbra = 8.99*sin_sa;
	double hf_umbra = 3.08*cos_sunangle_z;

	if (cone_umbra > hf_umbra) heat_input_tps = (hf_umbra + cone_umbra - z_sw)*solflux*tps_absorbtion*step_time;
	else heat_input_tps = (hf_umbra + hf_umbra - z_sw)*solflux*tps_absorbtion*step_time;
	heat_input_tps = heat_input_tps+(RTG_INPUT*step_time);

	double heat_input_x = 1.68*sin_sa*solflux*hull_absorbtion;
	double heat_input_zneg = 1.13*cos_sunangle_z*solflux*hull_absorbtion;
	if (z_sw == 0) heat_input_zneg = 0;

	heat_input_hull = step_time*(heat_input_x + heat_input_zneg);
	if (z_sw == 0 && cos_sunangle_z > 0.9962) heat_input_hull = 0;
	heat_input_hull = heat_input_hull+(RTG_INPUT*step_time);

	bus_th = heat_input_hull/step_time;
	tps_th = heat_input_tps/step_time;
	
	// cooling
	double hull_area = 6.41;
	double tps_area = 28.9;
	double tps_bot_area = 5;

	double hull_emis = 0.05;
	double tps_emis = 0.1;
	double tps_bot_emis = 0.005;

	double hull_t_diff = hull_temp-3;
	double tps_t_diff = tps_temp-3;

	heat_output_hull = -hull_area*hull_emis*StefBolt*hull_t_diff*hull_t_diff*hull_t_diff*hull_t_diff*step_time;
	heat_output_tps = -((tps_area*tps_emis)+(tps_bot_area*tps_bot_emis))*StefBolt*tps_t_diff*tps_t_diff*tps_t_diff*tps_t_diff*step_time;

	bus_th = bus_th + (heat_output_hull/step_time);
	tps_th = tps_th + (heat_output_tps/step_time);

	hull_psv_tf = bus_th;
	tps_psv_tf = tps_th;

	//dtemp_hull = (heat_input_hull+heat_output_hull)/(SPEC_HEAT_CAP*HULL_MASS);
	//dtemp_tps = (heat_input_tps+heat_output_tps)/(SPEC_HEAT_CAP*TPS_MASS);

	//hull_temp = hull_temp+dtemp_hull;
	//if (hull_temp < 0) hull_temp = 0;
	//tps_temp = tps_temp+dtemp_tps;
	//if (tps_temp < 0) tps_temp = 0;

	double req_temp = 293;

	double power_diff = 0;
	double power_shunt_hull = 0;
	double power_shunt_tps = 0;

	double hull_temp_diff = req_temp-hull_temp;
	double tps_temp_diff = req_temp-tps_temp;

	double req_energy_hull = hull_temp_diff*SPEC_HEAT_CAP*HULL_MASS;
	double req_energy_tps = tps_temp_diff*SPEC_HEAT_CAP*TPS_MASS;

	double ta = oapiGetTimeAcceleration();
	if (ta > 999) ta = 10/ta;
	else ta = 1;

	req_power_hull = (req_energy_hull*0.01*ta)-((heat_input_hull+heat_output_hull)/step_time);
	req_power_tps = (req_energy_tps*0.01*ta)-((heat_input_tps+heat_output_tps)/step_time);

	int hull_mode = 1;
	int tps_mode = 1;

	if (req_power_hull > 0 && req_power_tps > 0)
	{

		if ((req_power_hull + req_power_tps) > MAX_HEATER)
		{
			double fct = MAX_HEATER/(req_power_hull + req_power_tps);
			req_power_hull = req_power_hull*fct;
			req_power_tps = req_power_tps*fct;
		}
		hull_mode = 1;
		tps_mode = 1;
	}

	else if(req_power_hull > 0 && req_power_tps < 0)
	{
	power_diff = req_power_hull + req_power_tps;
	if (power_diff > 0)
		{
		req_power_hull = power_diff;
		power_shunt_hull =  -req_power_tps;
		power_shunt_tps = req_power_tps;
		req_power_tps = 0;
		}

	else if (power_diff < 0)
		{
		req_power_tps = power_diff;
		power_shunt_hull = req_power_hull;
		power_shunt_tps = -req_power_hull;
		req_power_hull = 0;
		}
	else
		{
		power_shunt_hull = req_power_hull;
		power_shunt_tps = req_power_tps;
		req_power_hull = 0;
		req_power_tps = 0;
		}

	if (req_power_hull > MAX_HEATER) req_power_hull = MAX_HEATER;
	if (req_power_tps < MAX_RADIATOR_TPS) req_power_tps = MAX_RADIATOR_TPS;


	hull_mode = 1;
	tps_mode = -1;
	}

	else if(req_power_hull < 0 && req_power_tps < 0)
	{
	if (req_power_hull < MAX_RADIATOR) req_power_hull = MAX_RADIATOR;
	if (req_power_tps < MAX_RADIATOR_TPS) req_power_tps = MAX_RADIATOR_TPS;
		hull_mode = -1;
		tps_mode = -1;
	}

	else if(req_power_hull < 0 && req_power_tps > 0)
	{
	power_diff = req_power_hull + req_power_tps;
	
	if (power_diff < 0)
		{
		req_power_hull = power_diff;
		power_shunt_hull =  -req_power_tps;
		power_shunt_tps = req_power_tps;
		req_power_tps = 0;
		}
	else if (power_diff > 0)
		{
		req_power_tps = power_diff;
		power_shunt_hull = req_power_hull;
		power_shunt_tps = -req_power_hull;
		req_power_hull = 0;
		}
	else
		{
		power_shunt_hull = req_power_hull;
		power_shunt_tps = req_power_tps;
		req_power_hull = 0;
		req_power_tps = 0;
		}

	if (req_power_hull < MAX_RADIATOR) req_power_hull = MAX_RADIATOR;
	if (req_power_tps > MAX_HEATER) req_power_tps = MAX_HEATER;


	hull_mode = -1;
	tps_mode = 1;
	}

	dtemp_hull = (heat_input_hull+heat_output_hull)/(SPEC_HEAT_CAP*HULL_MASS);
	dtemp_tps = (heat_input_tps+heat_output_tps)/(SPEC_HEAT_CAP*TPS_MASS);

	hull_temp = hull_temp+dtemp_hull;
	tps_temp = tps_temp+dtemp_tps;

	dtemp_hull = (step_time*req_power_hull)/(SPEC_HEAT_CAP*HULL_MASS);
	dtemp_tps = (step_time*req_power_tps)/(SPEC_HEAT_CAP*TPS_MASS);

	hull_temp = hull_temp+dtemp_hull;
	tps_temp = tps_temp+dtemp_tps;

	dtemp_hull = step_time*power_shunt_hull/(SPEC_HEAT_CAP*HULL_MASS);
	dtemp_tps = step_time*power_shunt_tps/(SPEC_HEAT_CAP*TPS_MASS);

	ps_bus_tps = power_shunt_hull;

	bus_th = bus_th + req_power_hull + power_shunt_hull;
	tps_th = tps_th + req_power_tps + power_shunt_tps;

	hull_temp = hull_temp+dtemp_hull;
	tps_temp = tps_temp+dtemp_tps;

	if (hull_temp > 1000000)hull_temp = 1000000;
	if (tps_temp > 1000000)tps_temp = 1000000;
	if (hull_temp < 0)hull_temp = 0;
	if (tps_temp < 0)tps_temp = 0;

	hud_upd_timer = hud_upd_timer + step_time;
	if (hud_upd_timer > 1)
	{
	hud_dtemp_hull = (hull_temp - old_hull_temp)/hud_upd_timer;
	old_hull_temp = hull_temp;
	hud_dtemp_tps = (tps_temp - old_tps_temp)/hud_upd_timer;
	old_tps_temp = tps_temp;
	hud_upd_timer = 0;
	}
	 
	
	//sprintf(oapiDebugString(), "i%0.4f  o%0.4f  i%0.4f  o%0.4f", hlin, hlot, tpin, tpot);
	//sprintf(oapiDebugString(), "%0.4f %0.4f %0.4f", cos_sunangle_x, cos_sunangle_y, cos_sunangle_z);
}


void solar_probe_2005::SetMeshMat()
{
double red_fact = 1-((1800-tps_temp)/1000);
double yel_fact = 0;
if (red_fact > 1)
{
	
	yel_fact = red_fact-1;
	red_fact = 1;
}
if (yel_fact > 1) yel_fact = 1;
if (yel_fact < 0) yel_fact = 0;
if (red_fact < 0) red_fact = 0;

MATERIAL *mat2 = oapiMeshMaterial (mesh12, 8);

mat2->emissive.r = red_fact*0.5;
mat2->emissive.g = yel_fact*0.05;
mat2->emissive.b = 0;
mat2->emissive.a = 1;


oapiSetMaterial(dhMesh2,8, mat2);
}

void solar_probe_2005::SetBusMeshMat()
{
double red_fact = 1-((1800-hull_temp)/1000);
double yel_fact = 0;
if (red_fact > 1)
{
	
	yel_fact = red_fact-1;
	red_fact = 1-yel_fact;
}
if (yel_fact > 1) yel_fact = 1;
if (yel_fact < 0) yel_fact = 0;
if (red_fact < 0) red_fact = 0;

for(int i=0;i<8;i++)
	{
MATERIAL *mat2 = oapiMeshMaterial (mesh12, i);

mat2->emissive.r = red_fact*0.5;
mat2->emissive.g = yel_fact*0.05;
mat2->emissive.b = 0;
mat2->emissive.a = 1;

mat2->ambient.r = 0;
mat2->ambient.g = 0;
mat2->ambient.b = 0;

mat2->diffuse.r = 0;
mat2->diffuse.g = 0;
mat2->diffuse.b = 0;


oapiSetMaterial(dhMesh2,i, mat2);
	}
}

void solar_probe_2005::DoSunPointing()
{
	VECTOR3 avel = _V(0,0,0);
	GetAngularVel(avel);
	double dt = oapiGetSimStep();
	double max_acc = 2*RAD;
	double drot = RAD*2*dt;

	char cbuf[256]="Sun";
	OBJHANDLE hRef = oapiGetGbodyByName (cbuf);
	VECTOR3 sun_pos_global = _V(0,0,0);
	oapiGetGlobalPos(hRef, &sun_pos_global);

	char abuf[256]="Earth";
	OBJHANDLE eRef = oapiGetGbodyByName (abuf);
	VECTOR3 earth_pos_global = _V(0,0,0);
	oapiGetGlobalPos(eRef, &earth_pos_global);

	sun_pos_global.x = sun_pos_global.x +70000000;
	sun_pos_global.y = sun_pos_global.y +70000000;
	sun_pos_global.z = sun_pos_global.z +70000000;

	VECTOR3 sun_dir_local = _V(0,0,0);
	Global2Local(sun_pos_global,sun_dir_local);

	VECTOR3 earth_dir_local = _V(0,0,0);
	Global2Local(earth_pos_global,earth_dir_local);

	VECTOR3 norm_ess_local = _V(0,0,0);
	norm_ess_local = crossp(earth_dir_local, sun_dir_local);

	VECTOR3 yup_local = _V(0,0,0);
	yup_local = crossp(sun_dir_local, norm_ess_local);


	double xaxis_angle = atan2(norm_ess_local.y,norm_ess_local.x);
	if (xaxis_angle < 0)xaxis_angle = xaxis_angle + (2*PI);

	double zaxis_angle = atan2(sun_dir_local.x,sun_dir_local.z);
	if (zaxis_angle < 0)zaxis_angle = zaxis_angle + (2*PI);

	double yaxis_angle = atan2(-yup_local.z,yup_local.y);
	if (yaxis_angle < 0)yaxis_angle = yaxis_angle + (2*PI);

	double max_dec_ang = 0;
	double max_rot = 2*RAD;


	if (xaxis_angle < PI)
	{
	max_dec_ang = (avel.z/max_acc)*avel.z*0.5;
	if (max_dec_ang > xaxis_angle)
	{	
		if (avel.z < 0)avel.z = avel.z+drot;
		else avel.z = avel.z-drot;
	}
	else if (avel.z > -max_rot)avel.z = avel.z-drot;
	else avel.z = avel.z+drot;
		
	}
	else
	{
	max_dec_ang = (avel.z/max_acc)*avel.z*0.5;
	if (max_dec_ang > ((2*PI)-xaxis_angle))
		{	
		if (avel.z > 0)avel.z = avel.z-drot;
		else avel.z = avel.z+drot;
		}
	else if (avel.z < max_rot)avel.z = avel.z+drot;
	else avel.z = avel.z-drot;
	}



	if (yaxis_angle < PI)
	{
	max_dec_ang = (avel.x/max_acc)*avel.x*0.5;
	if (max_dec_ang > yaxis_angle)
		{	
		if (avel.x < 0)avel.x = avel.x+drot;
		else avel.x = avel.x-drot;
		}
	else if (avel.x < max_rot)avel.x = avel.x+drot;
	else avel.x = avel.x-drot;
	}
	else
	{
	max_dec_ang = (avel.x/max_acc)*avel.x*0.5;
	if (max_dec_ang > ((2*PI)-yaxis_angle))
		{	
		if (avel.x > 0)avel.x = avel.x-drot;
		else avel.x = avel.x+drot;
		}
	else if (avel.x > -max_rot)avel.x = avel.x-drot;
	else avel.x = avel.x+drot;
	}



	if (zaxis_angle < PI)
	{
	max_dec_ang = (avel.y/max_acc)*avel.y*0.5;
	if (max_dec_ang > zaxis_angle)
		{	
		if (avel.y < 0)avel.y = avel.y+drot;
		else avel.y = avel.y-drot;
		}
	else if (avel.y > -max_rot)avel.y = avel.y-drot;
	else avel.y = avel.y+drot;
	}
	else
	{
	max_dec_ang = (avel.y/max_acc)*avel.y*0.5;
	if (max_dec_ang > ((2*PI)-zaxis_angle))
		{	
		if (avel.y > 0)avel.y = avel.y-drot;
		else avel.y = avel.y+drot;
		}
	else if (avel.y < max_rot)avel.y = avel.y+drot;
	else avel.y = avel.y-drot;
	}

	SetAngularVel(avel);
	double ang_dif_x = xaxis_angle;
	if (ang_dif_x > PI)ang_dif_x = (2*PI)-xaxis_angle;

	double ang_dif_y = yaxis_angle;
	if (ang_dif_y > PI)ang_dif_y = (2*PI)-yaxis_angle;

	double ang_dif_z = zaxis_angle;
	if (ang_dif_z > PI)ang_dif_z = (2*PI)-zaxis_angle;

	
	if ((drot*5) > ang_dif_x && (drot*5) > ang_dif_y && (drot*5) > ang_dif_z)
	{


	VECTOR3 yup = _V(0,0,0);
	VECTOR3 zforward = _V(0,0,0);
	VECTOR3 xright = _V(0,0,0);

	GlobalRot(norm_ess_local, xright);
	GlobalRot(yup_local, yup);
	GlobalRot(sun_dir_local,zforward);
	
	normalise(zforward);
	normalise(yup);
	normalise(xright);
	
	double gamma = atan2( xright.x, yup.x);
	double beta = -1*(asin(zforward.x));
	double alpha = atan2(zforward.y, zforward.z);
	SetGlobalOrientation (_V(alpha, beta, (PI/2)-gamma));
	SetAngularVel (_V(0,0,0));

	//sprintf(oapiDebugString(), "yup %0.4f %0.4f %0.4f", yup.x, yup.y, yup.z);
	//sprintf(oapiDebugString(), "g %0.4f %0.4f", xright.x, yup.x);
	//sprintf(oapiDebugString(), "x y %0.3f %0.3f %0.3f %0.3f %0.3f %0.3f %0.3f %0.3f %0.3f", xright.x, xright.y, xright.z, yup.x, yup.y, yup.z, zforward.x, zforward.y, zforward.z);
	}
	//sprintf(oapiDebugString(), "yup %0.4f %0.4f %0.4f", xaxis_angle*DEG, yaxis_angle*DEG, zaxis_angle*DEG);
}

void solar_probe_2005::DoBoom()
{
	SetThrusterLevel(th_boom, 1);
	//SetEnableFocus(false);

	for(int i=0;i<12;i++)
	{
	DelThruster(th_rcs[i]);
	}

	boom_sw = 1;
	asp_status = 0;
	kr_status = 0;

for(int i=0;i<8;i++)
	{
MATERIAL *mat2 = oapiMeshMaterial (mesh12, i);

mat2->ambient.r = 0;
mat2->ambient.g = 0;
mat2->ambient.b = 0;

mat2->diffuse.r = 0;
mat2->diffuse.g = 0;
mat2->diffuse.b = 0;


oapiSetMaterial(dhMesh2,i, mat2);
	}
}